%   ELAB_SONOGRAMA.M
%   Entradas:
%       PHANTOM_DATA.MAT: Posiciones de los dispersores de precompresión y
%           postcompresión.
%       PHANTOM_PARAMETERS.MAT: Contiene los otros datos del phantom.
%    
%   Autor: Daniel Serra Freese
%   Web: www.dranis.es
%   e-mail: danielserrafreese@gmail.com
%
%   Ultima resvision 7 septiembre 2016

%% Notas
%%% VER EL ERROR
% ??? Undefined function or variable 'env_pre'.
% 
% Error in ==> elab_sonograma at 213
% env_dB_pre=20*log10(env_pre);
% 
% Error in ==> elab_init at 75
% elab_sonograma.m
 


%% Parametros para FIELD II
f0=7e6;                 % Transducer center frequency [Hz]
fs=100e6;               % Sampling frequency [Hz]        
c=1540;                 % Speeposd of sound [m/s]
lambda=c/f0;            % Wave length [m]    
width=0.5/1000;           % Width of element
element_height=5/1000;  % Height of element [m]
kerf=width/20;          % Kerf [m]    
focus=[0 0 50]/1000;    % Fixed focal point [m]        
N_elements=192;           % Number of elements in the transducer    
N_active=64;            % Active elements in the transducer

%% Carga de datos
load phantom_data.mat
load phantom_parameters.mat


%% Configuracion y definicion de elementos
% Frecuencia de muestreo
set_sampling(fs);
set_field('show_times',0)

% Apertura de emision
emit_aperture = xdc_linear_array (N_elements, width, element_height, kerf, 1, 5, focus);

% Respuesta al impulso para el transductor
impulse_response=sin(2*pi*f0*(0:1/fs:2/f0));
impulse_response=impulse_response.*hanning(max(size(impulse_response)))';
xdc_impulse (emit_aperture, impulse_response);

% Excitación para la apertura de emision
excitation=sin(2*pi*f0*(0:1/fs:2/f0));
xdc_excitation (emit_aperture, excitation);

% Representamos apertura de emision
% figure,
% xdc_plot(emit_aperture)

% Apertura para recepcion
receive_aperture = xdc_linear_array (N_elements, width, element_height, kerf, 1, 5, focus);

% Respuesta al impulso para el transductor de recepcion
xdc_impulse (receive_aperture, impulse_response);


%% Calculo de las lineas de RF

% Numero de lineas de la imagen
no_lines=N_elements-N_active+1;% Number of A-lines in image

dx=width;% Incremento para imagen
z_focus=50/1000;

image_pre=zeros(1,no_lines);   % Pre-allocate some storage
image_post=zeros(1,no_lines);

% Comprobamos que la carpeta de datos existe. Si no existe la creamos.
if ~exist('rf_data','dir')
    disp('Creando carpeta rf_data...')
    mkdir 'rf_data'
    disp('Hecho!')
end

% Realizamos la simulacion. Permite interrumpir el proceso y retomarlo
% posteriormente o realizar el calculo en otro ordenador.

for i=1:no_lines
    % Mostramos progreso actual
        % Posicion para generar la imagen
        x=(i-1-no_lines/2)*dx;
        
        % Fijar posicion del focus
        xdc_center_focus (emit_aperture, [x 0 0]);
        xdc_focus (emit_aperture, 0, [x 0 z_focus]);
        xdc_center_focus (receive_aperture, [x 0 0]);
        xdc_focus (receive_aperture, 0, [x 0 z_focus]);
        
        % Fijamos los elmentos activos mediante la apodizacion
        apo=[zeros(1, i-1) hamming(N_active)' zeros(1, N_elements-N_active-i+1)];
        xdc_apodization (emit_aperture, 0, apo);
        xdc_apodization (receive_aperture, 0, apo);
        

    if ~exist(['rf_data/rfline_pre_',num2str(i),'.mat'])
        disp(['Calculando linea ' num2str(i) ' de ' num2str(no_lines) '... (PRE)'])
        
        % Calcula
        [rfline_pre, tline_pre]=calc_scat(emit_aperture, receive_aperture, phantom_pre, phantom_amp);
        
        %  Guardamos resultado
        cmd=['save rf_data/rfline_pre_',num2str(i),'.mat rfline_pre tline_pre'];
        eval(cmd);
        disp('Hecho!')
    end

    
    if ~exist(['rf_data/rfline_post_',num2str(i),'.mat'])
        disp(['Calculando linea ' num2str(i) ' de ' num2str(no_lines) '... (POST)'])
        [rfline_post, tline_post]=calc_scat(emit_aperture, receive_aperture, phantom_post, phantom_amp);
        
        %  Guardamos resultado
        cmd=['save rf_data/rfline_post_',num2str(i),'.mat rfline_post tline_post'];
        eval(cmd)
        disp('Hecho!')
    end
end

% Liberamos aperturas
xdc_free (emit_aperture)
xdc_free (receive_aperture)


%% Generar matriz de lineas rf a partir de las lineas guardadas
% Como las lineas han sido guardadas en archivos individuales se recompone
% la matriz con todas las lineas de rf.

lineLengths = [];

% Hacemos un whois a los archivos y guardamos las longitudes en un vector
% para posteriormente obtener la mas larga y dimensionar la matriz. Se
% realiza para cada conjunto de lineas pre y postcompresion.

for i=1:no_lines
    fileName = ['rf_data/rfline_pre_' num2str(i) '.mat'];
    s_pre = whos('rfline_pre', '-file',fileName );
    lineLengths_pre=[lineLengths s_pre.size];
    
    fileName = ['rf_data/rfline_post_' num2str(i) '.mat'];
    s_post = whos('rfline_post', '-file',fileName );
    lineLengths_post=[lineLengths s_post.size];
end

maxLength_pre = max(lineLengths_pre);
maxLength_post = max(lineLengths_post);

% Inicializamos matrices
rfAllLines_pre = zeros(maxLength_pre,no_lines);
rfAllLines_post = zeros(maxLength_post,no_lines);

tlineAll_pre = [];
tlineAll_post = [];

for i=1:no_lines
   fileName = ['rf_data/rfline_pre_' num2str(i) '.mat'];
   cmd = ['load ', fileName];
   eval(cmd);
   rfAllLines_pre(1:max(size(rfline_pre)),i)=rfline_pre;
   tlineAll_pre = [tlineAll_pre tline_pre];
   
   fileName = ['rf_data/rfline_post_' num2str(i) '.mat'];
   cmd = ['load ', fileName];
   eval(cmd);
   rfAllLines_post(1:max(size(rfline_post)),i)=rfline_post;
   tlineAll_post = [tlineAll_post tline_post];
end
 
% Minimo tiempo para primera muestra
minSample_pre=min(tline_pre)*fs;
minSample_post=min(tline_post)*fs;

% Alineamos en tiempo. El programa FIELD II devuelve en la primera variable
% el tiempo para la primera muestra de la senal obtenida del phantom. Al no
% no ser igual para todas las lineas es necesario que  realicemos una
% alineacion de todas.

for i=1:no_lines
    rfSingleAlignedLine = [];
    rfSingleAlignedLine = [zeros(round(tlineAll_pre(i)*fs-minSample_pre),1); rfAllLines_pre(:,i)];
    rfAllLinesAligned_pre(1:size(rfSingleAlignedLine,1),i) = rfSingleAlignedLine;
   
    rfSingleAlignedLine = [zeros(round(tlineAll_post(i)*fs-minSample_post),1); rfAllLines_post(:,i)];
    rfAllLinesAligned_post(1:size(rfSingleAlignedLine,1),i) = rfSingleAlignedLine;
    rfSingleAlignedLine = [];
end



%% Trazas individuales superpuestas (No necesario)
% Trazas individuales para cada conjunto de elementos.
figure
hold on

[N,M]=size(rfAllLinesAligned_pre);
for i=1:M
plot((0:N-1)/fs,(rfAllLinesAligned_pre(:,i)/max(max(rfAllLinesAligned_pre(:,i))))+i,'b'), hold on
end
axis tight, hold on, title('Senales RF del phantom de PRE- y POSTCompresion');

[N,M]=size(rfAllLines_post);
for i=1:M
plot((0:N-1)/fs,(rfAllLines_post(:,i)/max(max(rfAllLines_post(:,i))))+i,'r'), hold on
end
axis tight, hold off, title('Senales RF del phantom de PRE- y POSTCompresion')
 

%% Representar sonogramas(No necesario)
% Realizamos una compresion logaritmica a 60 dB.

 


  %  Encontrar envolvente ABS>HILBERT
    env_pre=abs(hilbert(rfAllLines_pre));
    env_post=abs(hilbert(rfAllLines_post));
 

figure
subplot(1,2,1)
env_dB_pre=20*log10(env_pre);
env_dB_pre=env_dB_pre-max(max(env_dB_pre));
env_gray_pre=127*(env_dB_pre+60)/60;

depth=((0:size(env_pre,1)-1))/fs*c/2;
x=((1:no_lines)-no_lines/2)*dx;

image(x*1000, depth*1000, env_gray_pre)
xlabel('Profundidad [mm]')
axis('image')
colormap(gray(128))
title('PREcompresion')

subplot(1,2,2)
env_dB_post=20*log10(env_post);
env_dB_post=env_dB_post-max(max(env_dB_post));
env_gray_post=127*(env_dB_post+60)/60;
depth=((0:size(env_post,1)-1))/fs*c/2;
x=((1:no_lines)-no_lines/2)*dx;
image(x*1000, depth*1000, env_gray_post)
xlabel('Profundidad [mm]')
axis('image')
colormap(gray(128))
title('POSTcompresion')


%% Datos de salida

% Guardamos las matrices de los sonogramas (lineas rf alineadas)
save sonogram_data.mat rfAllLinesAligned_pre rfAllLinesAligned_post

% Guaramos los datos necesarios para el calculo del elastograma.
save sonogram_parameters.mat fs f0 c no_lines 


%% Write info file

fid = fopen('info.txt', 'a');
info_00 = 'Informacion de la simulacion:';
info_01 = ['Frecuencia central del transductor [Hz]: ', num2str(f0)];
info_02 = ['Frecuencia de muestreo [Hz]: ',num2str(compresion*1000*dz)];
info_03 = ['Velocidad del sonido [m/s]: ', num2str(round(x_size*1000))];
info_04 = ['Longitud de onda [m]: ', num2str(y_size*1000)];
info_05 = ['Ancho de cada elemento [m]: ', num2str(z_size*1000)];
info_06 = ['Focus [X Y Z] [m]: ', num2str(dx)];
info_07 = ['Total de elmentos: ', num2str(dz)];
info_08 = ['Elementos activos: ', num2str(dz)];

fprintf(fid,info_00);
fprintf(fid,'\n\n\t');
fprintf(fid,info_01);
fprintf(fid,'\n\n\t');
fprintf(fid,info_02);
fprintf(fid,'\n\n\t');
fprintf(fid,info_03);
fprintf(fid,'\n\t');
fprintf(fid,info_04);
fprintf(fid,'\n\t');
fprintf(fid,info_05);
fprintf(fid,'\n\n\t');
fprintf(fid,info_06);
fprintf(fid,'\n\t');
fprintf(fid,info_07);
fprintf(fid,'\n\n');
fprintf(fid,info_08);
fprintf(fid,'\n\n');


fclose(fid);

clear all
